---- #########################################################################
---- #                                                                       #
---- # Copyright (C) OpenTX                                                  #
-----#                                                                       #
---- # License GPLv2: http://www.gnu.org/licenses/gpl-2.0.html               #
---- #                                                                       #
---- # This program is free software; you can redistribute it and/or modify  #
---- # it under the terms of the GNU General Public License version 2 as     #
---- # published by the Free Software Foundation.                            #
---- #                                                                       #
---- # This program is distributed in the hope that it will be useful        #
---- # but WITHOUT ANY WARRANTY; without even the implied warranty of        #
---- # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
---- # GNU General Public License for more details.                          #
---- #                                                                       #
---- #########################################################################

local defaultOptions = {
    { "Save", BOOL, 1 },
    { "Force", BOOL, 0 }
}

local latitude
local longitude
local saved_coordinates = {} 
local linesIterator = 0

local function getLogFileName()
    mdl = model.getInfo()
    filename = "/LOGS/" .. mdl["name"] .. "-last-gps-value.txt"
    return filename
end

local function rnd(v, d)
    if d then
        return math.floor((v*10^d)+0.5)/(10^d)
    else
        return math.floor(v+0.5)
    end
end

local function getTelemetryId(name)
    field = getFieldInfo(name)
    if field then
        return field.id
    else
        return -1
    end
end

local function init()
    gpsId = getTelemetryId("GPS")
end

local function save_last_pos_to_log(value_lat, value_lon)
    local log_file = io.open(getLogFileName(), "w")
    io.write(log_file, value_lat, "\r\n")
    io.write(log_file, value_lon, "\r\n")
    io.close(log_file)
end

local function load_last_pos_from_log()
    local dataBuffer
    local log_file

    log_file = io.open(getLogFileName(), "r")

    if log_file == nil then
        saved_coordinates[1] = "not available"
        saved_coordinates[2] = "not available"
    else
        dataBuffer = io.read(log_file, 512)
        io.close(log_file)

        linesIterator = 0

        for line in string.gmatch(dataBuffer, "([^\n]+)\n") do
            linesIterator = linesIterator + 1
            saved_coordinates[linesIterator] = line
        end
    end
end

local function create(zone, options)
    init()

    local gps_widget = { zone=zone, options=options, gps_value_lat="latitude", gps_value_lon="longitude", fromFile=false }

    if gps_widget.options.Save == 1 and gps_widget.options.Force == 1 then
        save_last_pos_to_log(0, 0)
    end

    return gps_widget
end

local function update(gps_widget, options)
    gps_widget.options = options
end

local function background(gps_widget)
    gpsLatLon = getValue(gpsId)

    if (type(gpsLatLon) == "table") then
        latitude = rnd(gpsLatLon["lat"], 6)
        longitude = rnd(gpsLatLon["lon"], 6)

--Save coordinates if is new value
        if gps_widget.options.Save == 1 and (gps_widget.gps_value_lat ~= latitude or gps_widget.gps_value_lon ~= longitude) then
            save_last_pos_to_log(latitude,longitude)
        end

        gps_widget.gps_value_lat = latitude
        gps_widget.gps_value_lon = longitude
        gps_widget.fromFile = false

    else
        if not gps_widget.fromFile then
            load_last_pos_from_log()
        end

        gps_widget.gps_value_lat = saved_coordinates[1]
        gps_widget.gps_value_lon = saved_coordinates[2]
        gps_widget.fromFile = true
    end

    return
end

local function refreshZoneMedium(gps_widget)
    lcd.drawRectangle(gps_widget.zone.x, gps_widget.zone.y, gps_widget.zone.w, gps_widget.zone.h, 0)
    lcd.drawRectangle(gps_widget.zone.x, gps_widget.zone.y, gps_widget.zone.w, 20, 0)

    if gps_widget.fromFile then
        lcd.drawText(gps_widget.zone.x+5, gps_widget.zone.y+2, "LAST SAVED POSITION:", SMLSIZE)
    else
        lcd.drawText(gps_widget.zone.x+5, gps_widget.zone.y+2, "POSITION NOW:", SMLSIZE)
    end

    lcd.drawText(gps_widget.zone.x+5, gps_widget.zone.y+25, "LAT: " .. gps_widget.gps_value_lat, 0)
    lcd.drawText(gps_widget.zone.x+5, gps_widget.zone.y+45, "LON: " .. gps_widget.gps_value_lon, 0)
    return
end

local function draw_error(gps_widget)
    lcd.drawText(gps_widget.zone.x, gps_widget.zone.y, "Not supported size", 0)
end

function refresh(gps_widget)
    background(gps_widget)

    if     gps_widget.zone.w > 380 and gps_widget.zone.h > 165 then draw_error(gps_widget)
    elseif gps_widget.zone.w > 180 and gps_widget.zone.h > 145 then draw_error(gps_widget)
    elseif gps_widget.zone.w > 170 and gps_widget.zone.h >  65 then refreshZoneMedium(gps_widget)
    elseif gps_widget.zone.w > 150 and gps_widget.zone.h >  28 then draw_error(gps_widget)
    elseif gps_widget.zone.w >  65 and gps_widget.zone.h >  35 then draw_error(gps_widget)
    end
end

return { name="GPSLog", options=defaultOptions, create=create, update=update, refresh=refresh, background=background }
